// Tests whether certain messages are received by a BView instance.
//
// Compile and execute with:
//
//     g++ -lbe -o ViewMessageTest ViewMessageTest.cpp
//     ./ViewMessageTest
//
// Please run this on a real BeOS instance and using the keyboard and mouse,
// try to check as many boxes as possible. Then post a screenshot with the 
// results.
//
// Simon South <simon@simonsouth.net>
// 15 August 2019

#include <stdio.h>

#include <map>
#include <vector>

#include <Application.h>
#include <CheckBox.h>
#include <View.h>
#include <Window.h>


#define MESSAGE_NAME_PAIR(message) pair<uint32,const char *>(message, #message)


class ViewMessageTestView : public BView {
	public:
								ViewMessageTestView(BRect frame);

	protected:
		void					MessageReceived(BMessage* message);

		map<uint32,BCheckBox*>	fMessageCheckBoxes;
};

class ViewMessageTestApp : public BApplication {
	public:
								ViewMessageTestApp();
	private:
		ViewMessageTestView*	fView;
		BWindow*				fWindow;
};

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ViewMessageTestView::ViewMessageTestView(BRect frame)
	:
	BView(frame, "View Message Test", B_FOLLOW_ALL_SIDES, B_WILL_DRAW),
	fMessageCheckBoxes()
{
	float y = 10.0;

	// The list of messages we're interested in capturing, together with their
	// name
	vector< pair<uint32,const char *> > messages;
	messages.reserve(8);
	messages.push_back(MESSAGE_NAME_PAIR(B_INVALIDATE));
	messages.push_back(MESSAGE_NAME_PAIR(B_KEY_DOWN));
	messages.push_back(MESSAGE_NAME_PAIR(B_KEY_UP));
	messages.push_back(MESSAGE_NAME_PAIR(B_MOUSE_DOWN));
	messages.push_back(MESSAGE_NAME_PAIR(B_MOUSE_MOVED));
	messages.push_back(MESSAGE_NAME_PAIR(B_MOUSE_UP));
	messages.push_back(MESSAGE_NAME_PAIR(B_UNMAPPED_KEY_DOWN));
	messages.push_back(MESSAGE_NAME_PAIR(B_UNMAPPED_KEY_UP));

	// Using this list, create a checkbox associated with each message
	for (vector< pair<uint32,const char *> >::iterator p = messages.begin();
			p != messages.end(); ++p) {
		const char *messageName = (*p).second;
		float height, width;

		BCheckBox *checkBox = new BCheckBox(messageName);
		checkBox->SetEnabled(false);
		checkBox->SetValue(0);
		checkBox->GetPreferredSize(&width, &height);
		checkBox->ResizeToPreferred();
		checkBox->MoveTo(10.0, y);

		fMessageCheckBoxes.insert(pair<uint32,BCheckBox*>((*p).first,
			checkBox));
		AddChild(checkBox);

		y += height + 3;
	}
}

void
ViewMessageTestView::MessageReceived(BMessage* message)
{
	// If the message is one we're listening for, check its box
	map<uint32,BCheckBox*>::iterator checkBoxPair =
		fMessageCheckBoxes.find(message->what);
	if (checkBoxPair != fMessageCheckBoxes.end())
		(*checkBoxPair).second->SetValue(1);

	BView::MessageReceived(message);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

ViewMessageTestApp::ViewMessageTestApp()
	:
	BApplication("application/x-vnd.simonsouth-viewmessagetest"),
	fView(NULL),
	fWindow(NULL)
{
	BRect frame(0, 0, 250, 250);

	fWindow = new BWindow(frame, "View Message Test", B_TITLED_WINDOW,
		B_QUIT_ON_WINDOW_CLOSE);

	fView = new ViewMessageTestView(frame);
	fWindow->AddChild(fView);

	fView->ResizeToPreferred();
	fView->MakeFocus(true);
	fView->Show();

	fWindow->CenterOnScreen();
	fWindow->Show();
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

int
main(int argc, char **argv)
{
	ViewMessageTestApp app;
	app.Run();

	return 0;
}
